@@ -33,7 +33,7 @@ Puppet::Type.type(:user).provide :user_role_add, :parent => :useradd, :source =>
     set("shell", value)
   end
 
-  has_features :manages_homedir, :allows_duplicates, :manages_solaris_rbac, :manages_passwords, :manages_password_age, :manages_shell
+  has_features :manages_homedir, :allows_duplicates, :manages_solaris_rbac, :manages_roles, :manages_passwords, :manages_password_age, :manages_shell
 
   def check_valid_shell
     unless File.exist?(@resource.should(:shell))
@@ -53,7 +53,8 @@ Puppet::Type.type(:user).provide :user_role_add, :parent => :useradd, :source =>
       # 1680 Now you can set the hashed passwords on solaris:lib/puppet/provider/user/user_role_add.rb
       # the value needs to be quoted, mostly because -c might
       # have spaces in it
-      if value = @resource.should(property) and value != ""
+      value = @resource.should(property)
+      if value && value != ""
         if property == :keys
           cmd += build_keys_cmd(value)
         else
@@ -99,7 +100,8 @@ Puppet::Type.type(:user).provide :user_role_add, :parent => :useradd, :source =>
       run(transition("normal"), "transition role to")
     else
       run(addcmd, "create")
-      if cmd = passcmd
+      cmd = passcmd
+      if cmd
         run(cmd, "change password policy for")
       end
     end
