@@ -132,7 +132,8 @@ Puppet::Type.newtype(:resources) do
 
   def resource_type
     unless defined?(@resource_type)
-      unless type = Puppet::Type.type(self[:name])
+      type = Puppet::Type.type(self[:name])
+      unless type
         raise Puppet::DevError, _("Could not find resource type")
       end
       @resource_type = type
@@ -174,7 +175,7 @@ Puppet::Type.newtype(:resources) do
     end
 
     # Otherwise, use a sensible default based on the OS family
-    @system_users_max_uid ||= case Facter.value(:osfamily)
+    @system_users_max_uid ||= case Puppet.runtime[:facter].value(:osfamily)
       when 'OpenBSD', 'FreeBSD'
         999
       else
